/* devicedetails.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <pwd.h>
#include "devicedetails.h"
#include "preferences.h"
#include "misc.h"
#include "main.h"
#include "features.h"
#include "gui.h"
#include "ipaddress.h"
#include "device.h"
#include "data.h"
#include "session.h"
#include "scheduler.h"
#include "licon.xpm"
#include "intup.xpm"
#include "intdown.xpm"
#include "lrouter.xpm"
#include "lswitch.xpm"
#include "lbridge.xpm"
#include "lhub.xpm"

static void select_row_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	if (row==0)
		gtk_clist_moveto(GTK_CLIST(clist), row, 0, 0.0, 0.0);
	else
	if (row==GTK_CLIST(clist)->rows - 1)
		gtk_clist_moveto(GTK_CLIST(clist), row, 0, 1.0, 0.0);
	else
	if (gtk_clist_row_is_visible(GTK_CLIST(clist), row-1)!=GTK_VISIBILITY_FULL)
		gtk_clist_moveto(GTK_CLIST(clist), row-1, 0, 0.0, 0.0);
	else
	if (gtk_clist_row_is_visible(GTK_CLIST(clist), row+1)!=GTK_VISIBILITY_FULL)
		gtk_clist_moveto(GTK_CLIST(clist), row+1, 0, 1.0, 0.0);
	gtk_clist_thaw(GTK_CLIST(clist));
	((DeviceDetails*)data)->select_row(row);
}
static void unselect_row_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{ ((DeviceDetails*)data)->unselect_row(row); }
static void ok_cb(GtkWidget *w, gpointer data)
{ ((DeviceDetails*)data)->save(); ((DeviceDetails*)data)->hide(); }
static void cancel_cb(GtkWidget *w, gpointer data)
{ ((DeviceDetails*)data)->hide(); }
static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((DeviceDetails*)data)->hide(); return TRUE; }

DeviceDetails::DeviceDetails(Main *m)
{
	GtkWidget *pixmap;
	GladeXML *gxml;

	main = m;
	device = NULL;

	gchar *fname = user_file("nomad.glade");
	gxml = glade_xml_new (fname, "devicedetails");
	if (!gxml)
	{
		g_warning("Failed to load file nomad.glade");
		return;
	}

	window = glade_xml_get_widget (gxml, "devicedetails");
	entry1 = glade_xml_get_widget (gxml, "entry1");
	entry2 = glade_xml_get_widget (gxml, "entry2");
	entry3 = glade_xml_get_widget (gxml, "entry3");
	entry4 = glade_xml_get_widget (gxml, "entry4");
	entry5 = glade_xml_get_widget (gxml, "entry5");
	entry6 = glade_xml_get_widget (gxml, "entry6");
	entry7 = glade_xml_get_widget (gxml, "entry7");
	entry17 = glade_xml_get_widget (gxml, "entry17");
	entry16 = glade_xml_get_widget (gxml, "entry16");
	checkbutton1 = glade_xml_get_widget (gxml, "checkbutton1");
	checkbutton2 = glade_xml_get_widget (gxml, "checkbutton2");
	button1 = glade_xml_get_widget (gxml, "button1");
	button2 = glade_xml_get_widget (gxml, "button2");
	frame2 = glade_xml_get_widget (gxml, "frame2");
	clist1 = glade_xml_get_widget (gxml, "clist1");
	clist2 = glade_xml_get_widget (gxml, "clist2");
	label14 = glade_xml_get_widget (gxml, "label14");
	label85 = glade_xml_get_widget (gxml, "label85");
	label86 = glade_xml_get_widget (gxml, "label86");
	label55 = glade_xml_get_widget (gxml, "label55");
	notebook2 = glade_xml_get_widget (gxml, "notebook2");
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

//	gtk_widget_set_usize (window, 420, 360);
	gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Device Details");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

  pixmap = create_pixmap(window, (char**) lrouter_xpm);
  gtk_widget_show (pixmap);
  gtk_container_add (GTK_CONTAINER (notebook2), pixmap);

  pixmap = create_pixmap(window, (char**) lswitch_xpm);
  gtk_widget_show (pixmap);
  gtk_container_add (GTK_CONTAINER (notebook2), pixmap);

  pixmap = create_pixmap(window, (char**) lbridge_xpm);
  gtk_widget_show (pixmap);
  gtk_container_add (GTK_CONTAINER (notebook2), pixmap);

  pixmap = create_pixmap(window, (char**) lhub_xpm);
  gtk_widget_show (pixmap);
  gtk_container_add (GTK_CONTAINER (notebook2), pixmap);

	gtk_signal_connect(GTK_OBJECT(clist1), "select_row",
		GTK_SIGNAL_FUNC(select_row_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist1), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_row_cb), this);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
		GTK_SIGNAL_FUNC (ok_cb), this);
  gtk_signal_connect (GTK_OBJECT (button2), "clicked",
		GTK_SIGNAL_FUNC (cancel_cb), this);
}

DeviceDetails::~DeviceDetails()
{
}

void DeviceDetails::empty()
{
	gtk_entry_set_text(GTK_ENTRY(entry1), "");
	gtk_entry_set_text(GTK_ENTRY(entry2), "");
	gtk_entry_set_text(GTK_ENTRY(entry3), "");
	gtk_entry_set_text(GTK_ENTRY(entry4), "");
	gtk_entry_set_text(GTK_ENTRY(entry5), "");
	gtk_entry_set_text(GTK_ENTRY(entry6), "");
	gtk_entry_set_text(GTK_ENTRY(entry7), "");
	gtk_entry_set_text(GTK_ENTRY(entry16), "");
	gtk_entry_set_text(GTK_ENTRY(entry17), "");
	gtk_frame_set_label(GTK_FRAME(frame2), "Interfaces");
	gtk_label_set_text(GTK_LABEL(label14), "");
	gtk_label_set_text(GTK_LABEL(label55), "");
	gtk_clist_clear(GTK_CLIST(clist1));
	gtk_clist_clear(GTK_CLIST(clist2));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton1), FALSE);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton2), FALSE);

	gtk_widget_set_sensitive(entry17, (device->subtype==device_merged)?TRUE:FALSE);
	gtk_widget_set_sensitive(entry16, (device->subtype==device_merged)?TRUE:FALSE);
	gtk_widget_set_sensitive(label85, (device->subtype==device_merged)?TRUE:FALSE);
	gtk_widget_set_sensitive(label86, (device->subtype==device_merged)?TRUE:FALSE);
}

void DeviceDetails::show()
{
	gchar *data[8];
	char buf[256];
	Interface *in;
	Link *ln;
	int i, row;

	gtk_widget_show(window);
	gdk_window_raise(window->window);

	empty();

	if (device!=NULL)
	{
		gtk_entry_set_text(GTK_ENTRY(entry1), device->name);
		gtk_entry_set_text(GTK_ENTRY(entry3), device->pingname.getstring());
		gtk_entry_set_text(GTK_ENTRY(entry2), device->community);
		gtk_entry_set_text(GTK_ENTRY(entry4), device->descr);
		gtk_entry_set_text(GTK_ENTRY(entry5), device->contact);
		gtk_entry_set_text(GTK_ENTRY(entry7), device->location);
		gtk_entry_set_text(GTK_ENTRY(entry6), device->comment);
		gtk_entry_set_text(GTK_ENTRY(entry17), device->pingname2.getstring());
		gtk_entry_set_text(GTK_ENTRY(entry16), device->community2);
		if (device->last_seen.zero()==0)
		{
			gtk_label_set_text(GTK_LABEL(label14), device->last_seen.date_to_str(1));
		}
		gtk_clist_freeze(GTK_CLIST(clist1));
		gtk_clist_clear(GTK_CLIST(clist1));

		row = 0;
		for (ln=device->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			if (in->type != 53)	// virtual interface
			{
				sprintf(buf, "%d", in->index); data[0] = strdup(buf);
				data[1] = strdup(in->descr);
				data[2] = strdup("");
				data[3] = strdup(in->mac.getstring());
				speed_to_str(in->speed, buf); data[4] = strdup(buf);
				data[5] = strdup("");
				data[6] = strdup("");
				interface_type_to_str(in->type, buf); data[7] = strdup(buf);
				gtk_clist_append(GTK_CLIST(clist1), data);
				delete data[0]; delete data[1]; delete data[2];
				delete data[3]; delete data[4]; delete data[5];
				delete data[6]; delete data[7];
				sprintf(buf, "%d", in->index);
				clist_pixtext(window, GTK_CLIST(clist1), row, 0,
					(gchar**)((in->status=='1')?intup_xpm:intdown_xpm), buf);
				gtk_clist_set_row_data(GTK_CLIST(clist1), row, in);
				row++;
			}
		}
		gtk_clist_thaw(GTK_CLIST(clist1));

		for (i=0; i<row; i++)
		{
			gtk_clist_set_background(GTK_CLIST(clist1), i, ((i%2)?(&main->gui->list_background_color_1):(&main->gui->list_background_color_2)));
		}

		sprintf(buf, "Interfaces (%d)", row);
		gtk_frame_set_label(GTK_FRAME(frame2), buf);

		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton1),
			(device->grab_top=='1')?TRUE:FALSE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton2),
			(device->grab_stats=='1')?TRUE:FALSE);
		// FIXME add extra pixmaps for device types
		if (device->subtype==device_router) i=1; else
		if (device->subtype==device_switch) i=2; else
		if (device->subtype==device_bridge) i=3; else
		if (device->subtype==device_hub) i=3; else
		if (device->subtype==device_merged) i=3; else
		if (device->subtype==device_placeholder) i=3; else i=4;
		gtk_notebook_set_page(GTK_NOTEBOOK(notebook2), i);
	}
}

void DeviceDetails::hide()
{
	gtk_widget_hide(window);
}

void DeviceDetails::save()
{
	if (device!=NULL)
	{
		strcpy(device->name, gtk_entry_get_text(GTK_ENTRY(entry1)));
		device->pingname.setstring(gtk_entry_get_text(GTK_ENTRY(entry3)));
		strcpy(device->community, gtk_entry_get_text(GTK_ENTRY(entry2)));
		strcpy(device->descr, gtk_entry_get_text(GTK_ENTRY(entry4)));
		strcpy(device->location, gtk_entry_get_text(GTK_ENTRY(entry7)));
		strcpy(device->comment, gtk_entry_get_text(GTK_ENTRY(entry6)));
		strcpy(device->contact, gtk_entry_get_text(GTK_ENTRY(entry5)));
		device->grab_top = (GTK_TOGGLE_BUTTON(checkbutton1)->active)?'1':'0';
		device->grab_stats = (GTK_TOGGLE_BUTTON(checkbutton2)->active)?'1':'0';
		main->session->devicebit=TRUE;
		main->setTitle();
	}
	main->gui->update_lists();
	main->gui->update_views();
}

void DeviceDetails::select_row(int row)
{
	Link *ln2;
	Interface *in;
	IPAddress *ip;
	char buf[256];
	gchar *data[1];

	in = (Interface*) gtk_clist_get_row_data(GTK_CLIST(clist1), row);
	gtk_label_set_text(GTK_LABEL(label55), in->comment);

	gtk_clist_clear(GTK_CLIST(clist2));
	for (ln2=in->ip_addresses.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		ip = (IPAddress*) ln2->item;
		strcpy(buf, ip->getstring());
		data[0] = strdup(buf);
		gtk_clist_append(GTK_CLIST(clist2), data);
		delete data[0];
	}
}

void DeviceDetails::unselect_row(int row)
{
	gtk_clist_clear(GTK_CLIST(clist2));
	gtk_label_set_text(GTK_LABEL(label55), "");
}

