/* gui.h
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef GUI_H
#define GUI_H 1

#include <gdk-pixbuf/gdk-pixbuf.h>
#include "list.h"

class Message;
class Main;
class Monitor;
class Discover;
class Print;
class Subnets;
class FilterDetails;
class Preferences;
class DeviceDetails;
class MachineDetails;
class NetworkDetails;
class TaskList;
class EventLog;
class Gui
{
	public:
		Gui(Main *m);
		~Gui();

		void update_views();
		void update_lists();
		void graph_window();
		void assign_color(gchar *config);
		void update_preferences();

		Main *main;
		Monitor *monitor;
		List stats;
		Print *print;
		Subnets *subnets;
		Discover *discover;
		Message *message;
		FilterDetails *filterdetails;
		Preferences *preferences;
		DeviceDetails *devicedetails;
		MachineDetails *machinedetails;
		NetworkDetails *networkdetails;
		TaskList *tasklist;
		EventLog *eventlog;
		GdkColor list_background_color_1, list_background_color_2;
};

void window_icon(GtkWidget *widget, char **filename);
GtkWidget *menu_icon(GtkWidget *widget, char **pix, char *label, GtkAccelGroup *grp);
GtkWidget *create_pixmap(GtkWidget *widget, char **filename);
void set_stipple(GtkWidget *widget, GdkGC *gc, char **filename);
GdkPixmap *load_pixmap(GtkWidget *widget, char *filename);
void clist_pixmap(GtkWidget *widget, GtkCList *clist, gint row, gint column, char **filename);
void clist_pixtext(GtkWidget *widget, GtkCList *clist, gint row, gint column, char **filename, gchar *str);
void alloc_color(GdkColor &col, guint16 r, guint16 g, guint16 b);
void speed_to_str(unsigned long spd, char *buf);
void seconds_to_time_string(unsigned long sec, char *buf);
void interface_type_to_str(int type, char *buf);

#endif

