/* networkdetails.cc
 * Copyright (C) 2001-2 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "networkdetails.h"
//#include "preferences.h"
#include "main.h"
#include "misc.h"
#include "machine.h"
#include "network.h"
#include "device.h"
#include "interface.h"
#include "session.h"
#include "gui.h"
#include "licon.xpm"
#include "srouter.xpm"
#include "sswitch.xpm"
#include "sserver.xpm"
#include "sprinter.xpm"
#include "scomputer.xpm"
#include "sbridge.xpm"
#include "shub.xpm"
#include "sunknown.xpm"
#include "intup.xpm"
#include "intdown.xpm"

static void close_cb(GtkWidget *w, gpointer data)
{ ((NetworkDetails*)data)->hide(); }
static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((NetworkDetails*)data)->hide(); return TRUE; }
static void machines_title_cb(GtkWidget *clist, gint column, gpointer data)
{ ((NetworkDetails*)data)->sort_machines(column); }
static gint clist_compare_ip(GtkCList *cl, gconstpointer p1, gconstpointer p2)
{
	GtkCListRow *row;
	Machine *m1, *m2;
	IPAddress ip1, ip2;

	row = (GtkCListRow*) p1;
	m1 = (Machine*) row->data;
	row = (GtkCListRow*) p2;
	m2 = (Machine*) row->data;
	ip1 = (m1->ip.zero())?m1->ipinv:m1->ip;
	ip2 = (m2->ip.zero())?m2->ipinv:m2->ip;
	return (ip1.compare(ip2));
}

static gint clist_compare_ts(GtkCList *cl, gconstpointer p1, gconstpointer p2)
{
	GtkCListRow *row;
	Machine *m1, *m2;

	row = (GtkCListRow*) p1;
	m1 = (Machine*) row->data;
	row = (GtkCListRow*) p2;
	m2 = (Machine*) row->data;
	return (m1->last_seen.compare(m2->last_seen));
}

NetworkDetails::NetworkDetails(Main *m)
{
	GladeXML *gxml;

	main = m;
	network = NULL;
	current_column = 0;

	gchar *fname = user_file("nomad.glade");
	gxml = glade_xml_new (fname, "networkdetails");
	if (!gxml)
	{
		g_warning("Failed to load file nomad.glade");
		return;
	}

	window = glade_xml_get_widget (gxml, "networkdetails");
	frame5 = glade_xml_get_widget (gxml, "frame5");
	frame6 = glade_xml_get_widget (gxml, "frame6");
	clist3 = glade_xml_get_widget (gxml, "clist3");
	clist4 = glade_xml_get_widget (gxml, "clist4");
	button6 = glade_xml_get_widget (gxml, "button6");
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

	gtk_widget_set_usize (window, 600, 300);
	gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Network Details");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

  gtk_signal_connect (GTK_OBJECT (button6), "clicked",
		GTK_SIGNAL_FUNC (close_cb), this);
  gtk_signal_connect (GTK_OBJECT (clist4), "click_column",
		GTK_SIGNAL_FUNC (machines_title_cb), this);
}

NetworkDetails::~NetworkDetails()
{
}

void NetworkDetails::empty()
{
	gtk_frame_set_label(GTK_FRAME(frame5), "Interfaces");
	gtk_frame_set_label(GTK_FRAME(frame6), "Machines");
	gtk_clist_clear(GTK_CLIST(clist3));
	gtk_clist_clear(GTK_CLIST(clist4));
}

void NetworkDetails::show()
{
	Machine *m;
	Interface *in;
	Link *ln, *ln2;
	char buf[128];
	gchar *data[7];
	char **img;
	int i, row;

	gtk_widget_show(window);
	gdk_window_raise(window->window);

	empty();

	if (network!=NULL)
	{
		row = 0;
		gtk_clist_freeze(GTK_CLIST(clist3));
		gtk_clist_clear(GTK_CLIST(clist3));
		for (ln=network->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			data[0] = strdup("");
			data[1] = strdup("");
			data[2] = strdup(in->descr);
			data[3] = strdup(in->mac.getstring());
			speed_to_str(in->speed, buf); data[4] = strdup(buf);
			interface_type_to_str(in->type, buf); data[5] = strdup(buf);
			gtk_clist_append(GTK_CLIST(clist3), data);
			delete data[0]; delete data[1]; delete data[2];
			delete data[3]; delete data[4]; delete data[5];

			// device
			strcpy(buf, "");
			if (in->device!=NULL) strcpy(buf, in->device->name);
			// FIXME add other device types
			if (in->device->subtype==device_router) img = srouter_xpm; else
			if (in->device->subtype==device_switch) img = sswitch_xpm; else
			if (in->device->subtype==device_bridge) img = sbridge_xpm; else
			if (in->device->subtype==device_hub) img = shub_xpm; else
				img = sunknown_xpm;
			clist_pixtext(window, GTK_CLIST(clist3), row, 0, (gchar **)img, buf);

			// index
			sprintf(buf, "%d", in->index);
			clist_pixtext(window, GTK_CLIST(clist3), row, 1,
				(gchar**)((in->status=='1')?intup_xpm:intdown_xpm), buf);
			row++;
		}
		gtk_clist_thaw(GTK_CLIST(clist3));

		for (i=0; i<row; i++)
		{
			gtk_clist_set_background(GTK_CLIST(clist3), i, ((i%2)?(&main->gui->list_background_color_1):(&main->gui->list_background_color_2)));
		}

		row = 0;
		gtk_clist_freeze(GTK_CLIST(clist4));
		gtk_clist_clear(GTK_CLIST(clist4));
		for (ln2=network->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			in = (Interface*) ln2->item;
			for (ln=in->machines.get_head(); ln!=NULL; ln=ln->next)
			{
				m = (Machine*) ln->item;
				data[0] = strdup("");
				data[1] = strdup(m->ip.getstring());
				data[2] = strdup(m->mac.getstring());
				data[3] = strdup(m->descr);
				data[4] = strdup(m->contact);
				data[5] = strdup(m->location);
				data[6] = strdup(m->last_seen.date_to_str(1));
				gtk_clist_append(GTK_CLIST(clist4), data);
				delete data[0]; delete data[1]; delete data[2];
				delete data[3]; delete data[4]; delete data[5];
				delete data[6];

				// name
				if (m->subtype==machine_printer) img = sprinter_xpm; else
				if (m->subtype==machine_server) img = sserver_xpm; else
					img = scomputer_xpm;
				clist_pixtext(window, GTK_CLIST(clist4), row, 0, (gchar **)img,
					m->name);
				gtk_clist_set_row_data(GTK_CLIST(clist4), row, m);

				row++;
			}
		}
		gtk_clist_thaw(GTK_CLIST(clist4));

		for (i=0; i<row; i++)
		{
			gtk_clist_set_background(GTK_CLIST(clist4), i, ((i%2)?(&main->gui->list_background_color_1):(&main->gui->list_background_color_2)));
		}
	}
}

void NetworkDetails::hide()
{
	gtk_widget_hide(window);
}

void NetworkDetails::sort_machines(gint col)
{
	current_column = col;
	gtk_clist_set_sort_column(GTK_CLIST(clist4), current_column);
	if (col==1)
		gtk_clist_set_compare_func (GTK_CLIST (clist4), clist_compare_ip);
	else
	if (col==6)
		gtk_clist_set_compare_func (GTK_CLIST (clist4), clist_compare_ts);
	else
		gtk_clist_set_compare_func (GTK_CLIST (clist4), NULL);
	gtk_clist_sort (GTK_CLIST(clist4));
}

