/* stats.h
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef STATS_H
#define STATS_H 1

#include <gtk/gtk.h>
#include "graphio.h"
#include "graph.h"

#define TOP_MGN 8
#define LEFT_MGN 56
#define RIGHT_MGN 4
#define BOTTOM_MGN 28

class Main;
class Interface;
class Device;
class DataSet;
class Stats
{
	public:
		Stats(Main *m);
		~Stats();

		void show();
		void hide();
		void nav_action(GtkWidget *w);
		void device_list();
		void set_interface_list();
//		void interface_list();
		void start_left();
		void start_right();
		void start_today();
		void start_all();
		void middle_left();
		void middle_right();
		void end_left();
		void end_right();

//		void set_check(GtkWidget *w);
		void key_color(GtkWidget *w, gchar *str);
		void create_key(GtkWidget *vbox, char *label, int i, GdkColor col);
		void load();
		void draw();
		void draw_init_canvas(GtkWidget *drawingarea, Graph *g);
		void draw_failed(GtkWidget *drawingarea, Graph *g);
		void draw_graph(GtkWidget *drawingarea, Graph *g);
		void draw_line(Graph *g, int var);

		Main *main;
		Graph *packets, *errors;
		gboolean packets_ok;
		gboolean errors_ok;

//		gint width1, height1;
//		gint width2, height2;
//		GdkPixmap *canvas1, *canvas2;
		int BX, BY, BW, BH;
		int length;			// number of days worth of data
		int day_range;	// the number of days to be shown
		TimeStamp gstart, gend;	// range of timestamps to be shown in the graph
		TimeStamp newest_time, oldest_time;

		GraphIO gio;
//		DataSet *data1, *data2;
//		char *check_states;
//		gboolean draw_lock;

		Device *current_router;
		Interface *current_interface;

		GdkGC *gc, *gridgc;
		GdkColor lightgray, gray, darkgray, black;

		GtkWidget *window;
		GtkWidget *navbox;
		GtkWidget *navbox2;
		GtkWidget *nbutton1;
		GtkWidget *npanel1;
		GtkWidget *nbutton2;
		GtkWidget *npanel2;
//		GtkWidget *clist5;
		GtkWidget *clist6;
		GtkWidget *combo3;

		GtkWidget *button_utilization;
		GtkWidget *button_packets_in;
		GtkWidget *button_packets_out;
		GtkWidget *button_broadcast_in;
		GtkWidget *button_broadcast_out;
		GtkWidget *button_multicast_in;
		GtkWidget *button_multicast_out;
		GtkWidget *button_errors_in;
		GtkWidget *button_errors_out;
		GtkWidget *label_utilization;
		GtkWidget *label_packets_in;
		GtkWidget *label_packets_out;
		GtkWidget *label_broadcast_in;
		GtkWidget *label_broadcast_out;
		GtkWidget *label_multicast_in;
		GtkWidget *label_multicast_out;
		GtkWidget *label_errors_in;
		GtkWidget *label_errors_out;

		GtkWidget *button_dropped;
		GtkWidget *button_crc;
		GtkWidget *button_runts;
		GtkWidget *button_giant;
		GtkWidget *button_fragments;
		GtkWidget *button_jabbers;
		GtkWidget *button_collisions;
		GtkWidget *label_dropped;
		GtkWidget *label_crc;
		GtkWidget *label_runts;
		GtkWidget *label_giant;
		GtkWidget *label_fragments;
		GtkWidget *label_jabbers;
		GtkWidget *label_collisions;

		GtkWidget *button1;
		GtkWidget *button2;
		GtkWidget *button3;
		GtkWidget *button4;
		GtkWidget *button5;
		GtkWidget *button6;
		GtkWidget *button13;
		GtkWidget *button15;
		GtkWidget *button16;
//		GtkWidget *checkbutton1;
//		GtkWidget *label46;
		GtkWidget *radio1;
		GtkWidget *radio2;
		GtkWidget *radio3;
		GtkWidget *radio4;
//		GtkWidget *vbox19;
		GtkWidget *drawingarea1;
		GtkWidget *drawingarea2;
		GtkWidget *scrollbar;
		GtkAdjustment *scrollbar_adj;
		GtkWidget *pix[8];
};

#endif

