/* subnets.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include "subnets.h"
#include "main.h"
#include "session.h"
#include "gui.h"
#include "data.h"
#include "licon.xpm"

static void close_cb(GtkWidget *w, gpointer data)
{ ((Subnets*)data)->hide(); }

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((Subnets*)data)->hide(); return (TRUE); }

static void select_cb(GtkWidget *clist, gint row, gint col, GdkEventButton *e, gpointer data)
{
/*
	SubnetView *m = (SubnetView*) data;
	Subnet *r = (Subnet*) gtk_clist_get_row_data(GTK_CLIST(m->clist7), row);
	gtk_entry_set_text(GTK_ENTRY(m->entry17), r->name);
	gtk_widget_set_sensitive(m->hbox36, TRUE);
	m->current = r;
	m->current_row = row;
*/
}

static void unselect_cb(GtkWidget *clist, gint row, gint col, GdkEventButton *e, gpointer data)
{
/*
	SubnetView *m = (SubnetView*) data;
	gtk_widget_set_sensitive(m->hbox36, FALSE);
	m->current = NULL;
*/
}

static void update_cb(GtkWidget *w, gpointer data)
{
/*
	SubnetView *s = (SubnetView*) data;
	Subnet *m = (Subnet*) s->current;
	strcpy(m->name, gtk_entry_get_text(GTK_ENTRY(s->entry17)));
	gtk_clist_set_text(GTK_CLIST(s->clist7), s->current_row, 0, m->name);
*/
}

static gint clist_compare_ip(GtkCList *cl, gconstpointer p1, gconstpointer p2)
{
	GtkCListRow *row;
	Subnet *sn, *s2;

	row = (GtkCListRow*) p1;
	sn = (Subnet*) row->data;
	row = (GtkCListRow*) p2;
	s2 = (Subnet*) row->data;

	return (sn->ip.compare(s2->ip));
}

static void title_cb(GtkWidget *clist, gint column, gpointer data)
{
	((Subnets*)data)->current_column = column;
	((Subnets*)data)->refresh();
}

Subnets::Subnets(Main *m)
{
	GtkWidget	*hseparator3;
	GtkWidget *hbox76;

	main = m;
	current = NULL;
	current_row = 0;
	current_column = 0;

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Subnets");
	gtk_widget_set_usize(window, 400, 180);
  gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Subnets");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy_event",
		GTK_SIGNAL_FUNC(delete_event), this);

  vbox22 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox22);
  gtk_container_set_border_width (GTK_CONTAINER (vbox22), 4);
  gtk_container_add (GTK_CONTAINER (window), vbox22);

  scrolledwindow8 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow8);
  gtk_box_pack_start (GTK_BOX (vbox22), scrolledwindow8, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow8),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  clist7 = gtk_clist_new (2);
  gtk_widget_show (clist7);
  gtk_container_add (GTK_CONTAINER (scrolledwindow8), clist7);
  gtk_clist_set_column_width (GTK_CLIST (clist7), 0, 120);
  gtk_clist_set_column_width (GTK_CLIST (clist7), 1, 240);
  gtk_clist_column_titles_show (GTK_CLIST (clist7));
	gtk_signal_connect(GTK_OBJECT(clist7), "select_row",
		GTK_SIGNAL_FUNC(select_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist7), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_cb), this);
  gtk_signal_connect (GTK_OBJECT (clist7), "click_column",
		GTK_SIGNAL_FUNC (title_cb), this);

  label40 = gtk_label_new ("IP Address/ Mask");
  gtk_widget_show (label40);
  gtk_clist_set_column_widget (GTK_CLIST (clist7), 0, label40);

  label41 = gtk_label_new ("Description");
  gtk_widget_show (label41);
  gtk_clist_set_column_widget (GTK_CLIST (clist7), 1, label41);

  hbox36 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox36);
  gtk_box_pack_start (GTK_BOX (vbox22), hbox36, FALSE, TRUE, 0);

  entry17 = gtk_entry_new ();
  gtk_widget_show (entry17);
  gtk_box_pack_start (GTK_BOX (hbox36), entry17, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(entry17), "activate",
		GTK_SIGNAL_FUNC(update_cb), this);

  button16 = gtk_button_new_with_label ("Update");
  gtk_widget_show (button16);
  gtk_box_pack_start (GTK_BOX (hbox36), button16, FALSE, FALSE, 0);
  gtk_widget_set_usize (button16, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button16), "clicked",
		GTK_SIGNAL_FUNC (update_cb), this);

	hseparator3 = gtk_hseparator_new ();
	gtk_widget_show (hseparator3);
	gtk_box_pack_start (GTK_BOX (vbox22), hseparator3, FALSE, TRUE, 0);

	hbox76 = gtk_hbox_new (TRUE, 4);
	gtk_widget_show (hbox76);
	gtk_box_pack_start (GTK_BOX (vbox22), hbox76, FALSE, TRUE, 0);

	button71 = gtk_button_new_with_label ("Close");
	gtk_widget_show (button71);
	gtk_box_pack_end (GTK_BOX (hbox76), button71, FALSE, FALSE, 0);
	gtk_widget_set_usize (button71, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button71), "clicked",
		GTK_SIGNAL_FUNC (close_cb), this);
}

Subnets::~Subnets()
{
}
void Subnets::show()
{
	gtk_widget_show(window);
	refresh();
	gdk_window_raise(window->window);
}

void Subnets::hide()
{
	gtk_widget_hide(window);
}

void Subnets::refresh()
{
	Link *ln;
	Subnet *sn;
	gchar *data[2];
	char buf[256];
	int i, row;

	if (main->session->data==NULL) return;

	row=0;
	current = NULL;
	gtk_clist_freeze(GTK_CLIST(clist7));
	gtk_clist_clear(GTK_CLIST(clist7));
	for (ln=main->session->data->subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		sn = (Subnet*) ln->item;
		sprintf(buf, "%s/%d", sn->ip.getstring(), sn->mask);
		data[0] = strdup(buf);
		data[1] = strdup(sn->name);
		gtk_clist_append(GTK_CLIST(clist7), data);
		delete data[0]; delete data[1];
		gtk_clist_set_row_data(GTK_CLIST(clist7), row, sn);
		row++;
	}
	gtk_widget_set_sensitive(hbox36, FALSE);
	sort(current_column);
	gtk_clist_thaw(GTK_CLIST(clist7));
}

void Subnets::sort(gint col)
{
	current_column = col;
	gtk_clist_set_sort_column(GTK_CLIST(clist7), current_column);
	if (current_column==0)
		gtk_clist_set_compare_func (GTK_CLIST (clist7), clist_compare_ip);
	else
		gtk_clist_set_compare_func (GTK_CLIST (clist7), NULL);
	gtk_clist_sort (GTK_CLIST(clist7));
	// printf("SORTED SUBNETS %d\n", col);
}

