/* graphio.h
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef HEADER_H
#define HEADER_H 1

#include "../config.h"

#define DATA_DIR ".nomad"
#define SHARE_DIR "/usr/share/nomad"

#define NUM_PROCESSES 4

#define GRAPH_MAX_BLOCKS 8
#define GRAPH_NUM_PACKETS 10
#define GRAPH_NUM_ERRORS 8
#define GRAPH_NUM_TOTAL (1+GRAPH_NUM_PACKETS+GRAPH_NUM_ERRORS)
#define GRAPH_FILE_HEADER_SIZE 8
#define GRAPH_BLOCK_HEADER_SIZE 29
#define GRAPH_PACKET_RECORD_SIZE 89
#define GRAPH_ERROR_RECORD_SIZE 65

#define MIN_SAVE_SESSION_TIME 120
#define MIN_PING_DEVICES_TIME 20
#define MIN_REGRAB_TOPOLOGY_TIME 600
#define MIN_IMPORT_TIME 120
#define MIN_STATS_TIME 120

#define CONFIG_TEST "/apps/nomad/gconf_test"
#define CONFIG_SESSION_STARTUP "/apps/nomad/session/startup"
#define CONFIG_SESSION_AUTO_SAVE_EVERY "/apps/nomad/session/auto_save_every"
#define CONFIG_SESSION_SAVE_SESSION_TIME "/apps/nomad/session/save_session_time"
#define CONFIG_SESSION_AUTO_SAVE_ON_QUIT "/apps/nomad/session/auto_save_on_quit"
#define CONFIG_SESSION_PING_TIME "/apps/nomad/session/ping_time"
#define CONFIG_SESSION_TOPOLOGY_TIME "/apps/nomad/session/topology_time"
#define CONFIG_SESSION_TOPOLOGY_DEFAULT "/apps/nomad/session/topology_default"
#define CONFIG_SESSION_GRAPH_DEFAULT "/apps/nomad/session/graph_default"
#define CONFIG_IMPORT_AUTO_IMPORT_FILES "/apps/nomad/import/auto_import_files"
#define CONFIG_IMPORT_IMPORT_TIME "/apps/nomad/import/import_time"
#define CONFIG_IMPORT_FILES "/apps/nomad/import/files"
#define CONFIG_SUPPORT_PING "/apps/nomad/support/ping"
#define CONFIG_SUPPORT_TELNET "/apps/nomad/support/telnet"
#define CONFIG_SUPPORT_TRACEROUTE "/apps/nomad/support/traceroute"
#define CONFIG_SEARCH_AUTO_ZOOM "/apps/nomad/search/auto_zoom"
#define CONFIG_SEARCH_AUTO_HIDE "/apps/nomad/search/auto_hide"
#define CONFIG_SEARCH_SHOW_HIDDEN "/apps/nomad/search/show_hidden"
#define CONFIG_SEARCH_SELECT_MATCHES "/apps/nomad/search/select_matches"
#define CONFIG_DIAGRAM_ANIMATE_ZOOM "/apps/nomad/diagram/animate_zoom"
#define CONFIG_DIAGRAM_SHOW_LABELS "/apps/nomad/diagram/show_labels"
#define CONFIG_DIAGRAM_SHOW_LINE_COLORS "/apps/nomad/diagram/show_line_colors"
#define CONFIG_DIAGRAM_SHOW_LEAF "/apps/nomad/diagram/show_leaf"
#define CONFIG_DIAGRAM_SHOW_EMPTY_LEAF "/apps/nomad/diagram/show_empty_leaf"
#define CONFIG_DIAGRAM_GROUP_LEAF "/apps/nomad/diagram/group_leaf"
#define CONFIG_DIAGRAM_NETWORK_TYPE "/apps/nomad/diagram/network_type"
#define CONFIG_DIAGRAM_NETWORK_POSITION "/apps/nomad/diagram/network_position"
#define CONFIG_DIAGRAM_SHOW_MACHINES "/apps/nomad/diagram/show_machines"
#define CONFIG_DIAGRAM_GROUP_MACHINES "/apps/nomad/diagram/group_machines"
#define CONFIG_DIAGRAM_FONT "/apps/nomad/diagram/font"
#define CONFIG_DIAGRAM_BACKGROUND "/apps/nomad/diagram/background"
#define CONFIG_DIAGRAM_DEVICE_UP "/apps/nomad/diagram/device_up"
#define CONFIG_DIAGRAM_DEVICE_DOWN "/apps/nomad/diagram/device_down"
#define CONFIG_DEVICE_UP_SELECTED "/apps/nomad/diagram/device_up_selected"
#define CONFIG_DEVICE_DOWN_SELECTED "/apps/nomad/diagram/device_down_selected"
#define CONFIG_DIAGRAM_MACHINE "/apps/nomad/diagram/machine"
#define CONFIG_DIAGRAM_MACHINE_SELECTED "/apps/nomad/diagram/machine_selected"
#define CONFIG_DIAGRAM_NETWORK "/apps/nomad/diagram/network"
#define CONFIG_DIAGRAM_NETWORK_EMPTY "/apps/nomad/diagram/network_empty"
#define CONFIG_DIAGRAM_NETWORK_SELECTED "/apps/nomad/diagram/network_selected"
#define CONFIG_DIAGRAM_LIST_BG_1 "/apps/nomad/diagram/list_background_color_1"
#define CONFIG_DIAGRAM_LIST_BG_2 "/apps/nomad/diagram/list_background_color_2"
#define CONFIG_COMMUNITY "/apps/nomad/community"
#define CONFIG_SAMPLE_FREQUENCY "/apps/nomad/sample/frequency"
#define CONFIG_SAMPLE_DURATION "/apps/nomad/sample/duration"
#define CONFIG_SAMPLE_ERRORS "/apps/nomad/sample/errors"
#define CONFIG_SAMPLE_RATE "/apps/nomad/sample/rate"
#define CONFIG_SAMPLE_REPLACE "/apps/nomad/sample/replace"
#define CONFIG_EMAIL_SENDER_ADDRESS "/apps/nomad/email/sender_address"
#define CONFIG_EMAIL_SMTP_SERVER "/apps/nomad/email/smtp_server"
#define CONFIG_EMAIL_PORT "/apps/nomad/email/port"
#define CONFIG_EMAIL_SUBJECT "/apps/nomad/email/subject"
#define CONFIG_EMAIL_BODY "/apps/nomad/email/body"
#define CONFIG_SHOW_SIDEBAR "/apps/nomad/show_sidebar"
#define CONFIG_SHOW_STATUSBAR "/apps/nomad/show_statusbar"

enum FilterType {
	filter_permit,
	filter_deny
};

enum VisualType {
	visual_device,
	visual_network,
	visual_machine
};

enum DeviceType {
	device_switch,
	device_router,
	device_bridge,
	device_hub,
	device_merged,
	device_placeholder,
	device_unknown
};

enum DeviceGroupType {
	device_group_bunch,
	device_group_star,
	device_group_none
};

enum MachineType {
	machine_computer,
	machine_printer,
	machine_server
};

enum NetworkType {
	network_vbus,
	network_hbus,
	network_p2p,
	network_ring,
	network_cloud,
	network_auto
};

enum NetworkPositionType {
	network_position_auto,
	network_position_manual
};

enum NetworkGroupType {
	network_group_bus,
	network_group_star,
	network_group_none,
	network_group_bunch
};

enum ActionType {
	action_send_email,
	action_play_sound,
	action_run_script
};

#endif

