#include <stdlib.h>
#include <string.h>
#include "machine.h"
#include "network.h"
#include "subnet.h"
#include "misc.h"

Machine::Machine()
{
	empty();
}

Machine::~Machine()
{
}

void Machine::empty()
{
	x = rand()%4000;
	y = rand()%3000;
	w = h = 96;
	selection = FALSE;
	shown = FALSE;
	maintype = visual_machine;
	subtype = machine_computer;

	strcpy(name, "");
	ip.empty();
	ipinv.empty();
	last_seen.empty();
	strcpy(location, "");
	strcpy(descr, "");
	strcpy(contact, "");
	mac.empty();
//	registered = '0';
	interface = NULL;
}

void Machine::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	char buf[16];

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Machine", NULL);
	if (strcmp(name, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"name", (xmlChar*)name);
	if (!ip.zero())
		xmlSetProp(newxml, (xmlChar*)"ip", (xmlChar*)ip.getstring());
	if (!ipinv.zero())
		xmlSetProp(newxml, (xmlChar*)"ip2", (xmlChar*)ipinv.getstring());
	if (!last_seen.zero())
		xmlSetProp(newxml, (xmlChar*)"lastseen", (xmlChar*)last_seen.getstring());
	if (strcmp(location, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"location", (xmlChar*)location);
	if (strcmp(descr, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"description", (xmlChar*)descr);
	if (strcmp(contact, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"contact", (xmlChar*)contact);
	if (!mac.zero())
		xmlSetProp(newxml, (xmlChar*)"mac", (xmlChar*)mac.getstring());
	if (subtype==machine_server) strcpy(buf, "server"); else
	if (subtype==machine_printer) strcpy(buf, "printer"); else
		strcpy(buf, "computer");
	xmlSetProp(newxml, (xmlChar*)"subtype", (xmlChar*)buf);
	sprintf(buf, "%d", x);
	xmlSetProp(newxml, (xmlChar*)"x", (xmlChar*)buf);
	sprintf(buf, "%d", y);
	xmlSetProp(newxml, (xmlChar*)"y", (xmlChar*)buf);
	sprintf(buf, "%s", (shown==TRUE)?"yes":"no");
	xmlSetProp(newxml, (xmlChar*)"show", (xmlChar*)buf);

}

void Machine::loadxml(xmlNodePtr xmlnode)
{
	xmlChar *buf;

	// check this is the correct  node
	if (!xmlnode ||!xmlnode->name || strcmp((char*)xmlnode->name, "Machine")!=0)
		return;
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"name");
	if (buf!=NULL) { strcpy(name, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"ip");
	if (buf!=NULL) { ip.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"ip2");
	if (buf!=NULL) { ipinv.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"lastseen");
	if (buf!=NULL) { last_seen.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"location");
	if (buf!=NULL) { strcpy(location, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"description");
	if (buf!=NULL) { strcpy(descr, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"contact");
	if (buf!=NULL) { strcpy(contact, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"mac");
	if (buf!=NULL) { mac.setstring((char*)buf); free(buf); }
	maintype = visual_machine;
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"subtype");
	if (buf!=NULL)
	{
		if (strcmp((char*)buf, "server")==0) subtype = machine_server; else
		if (strcmp((char*)buf, "printer")==0) subtype = machine_printer; else
			subtype = machine_computer;
		free(buf);
	}
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"x");
	if (buf!=NULL) { sscanf((char*)buf, "%d", &x); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"y");
	if (buf!=NULL) { sscanf((char*)buf, "%d", &y); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"show");
	if (buf!=NULL)
	{ shown = (strcmp((char*)buf, "yes")==0)?TRUE:FALSE; free(buf); }
}

int Machine::match(char *str)
{
	char nm[16];

	if (strlen(str)==0) return 0;

	if ((strlen(name)>0) && (lower_contains(name, str))) return 1;
	if ((strlen(location)>0) && (lower_contains(location, str))) return 2;
	if ((strlen(descr)>0) && (lower_contains(descr, str))) return 3;
	if ((strlen(contact)>0) && (lower_contains(contact, str))) return 4;

	sprintf(nm, "%s", ip.getstring());
	if (lower_contains(nm, str)) return 5;
	sprintf(nm, "%s", ipinv.getstring());
	if (lower_contains(nm, str)) return 6;
	sprintf(nm, "%s", mac.getstring());
	if (lower_contains(nm, str)) return 7;

	// also check lastseen to match on date (but not in quick search)

	return 0;
}

int Machine::matchsubnet(Subnet *s)
{
	if (ip.in_subnet(s->ip, s->mask)) return 1;
	if (ipinv.in_subnet(s->ip, s->mask)) return 2;

	return 0;
}

