#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "viewitem.h"
#include "view.h"
#include "device.h"
#include "network.h"
#include "interface.h"
#include "data.h"

View::View()
{
	strcpy(name, "");
	empty();
}

View::~View()
{
	strcpy(name, "");
	empty();
}

void View::empty()
{
	Link *ln;
	ViewItem *in;

	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (ViewItem*) ln->item;
		if (in!=NULL) delete in;
		ln->item = NULL;
	}
	devices.empty();
	for (ln = networks.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (ViewItem*) ln->item;
		if (in!=NULL) delete in;
		ln->item = NULL;
	}
	networks.empty();
	for (ln = machines.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (ViewItem*) ln->item;
		if (in!=NULL) delete in;
		ln->item = NULL;
	}
	machines.empty();
}

void View::save(gchar *fname)
{
	xmlDocPtr doc;
	ViewItem *in;
	Link *ln;

	doc = xmlNewDoc((xmlChar*)"1.0");
	doc->root = xmlNewDocNode(doc, NULL, (xmlChar*)"Nomad_View", NULL);
	if (strcmp(name, "")!=0)
		xmlSetProp(doc->root, (xmlChar*)"name", (xmlChar*)name);
	for (ln=devices.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (ViewItem*) ln->item;
		if (in!=NULL) in->savexml(doc->root, visual_device);
	}
	for (ln=networks.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (ViewItem*) ln->item;
		if (in!=NULL) in->savexml(doc->root, visual_network);
	}
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (ViewItem*) ln->item;
		if (in!=NULL) in->savexml(doc->root, visual_machine);
	}
	xmlSaveFile(fname, doc);
	xmlFreeDoc(doc);
}

void View::load(gchar *fname)
{
	ViewItem *in;
	Device *r;
	Network *n;
	Link *ln;
	xmlChar *buf;
	xmlDocPtr doc;
	xmlNodePtr xmlnode;

	strcpy(name, "");
	empty();

	doc = xmlParseFile((gchar*)fname);
	if (doc && doc->root && doc->root->name &&
			strcmp((gchar*)doc->root->name, "Nomad_View")==0)
	{
		buf = (xmlChar*)xmlGetProp(doc->root, (xmlChar*)"name");
		if (buf!=NULL) { strcpy(name, (char*)buf); free(buf); }
		xmlnode = doc->root->childs;
		while (xmlnode != NULL)
		{
			if (xmlnode->name!=NULL)
			{
				if (strcmp((char*)xmlnode->name, "Device")==0)
				{
					in = new ViewItem();
					in->loadxml(xmlnode);
					devices.tag(in);
				}else
				if (strcmp((char*)xmlnode->name, "Network")==0)
				{
					in = new ViewItem();
					in->loadxml(xmlnode);
					networks.tag(in);
				}else
				if (strcmp((char*)xmlnode->name, "Machine")==0)
				{
					in = new ViewItem();
					in->loadxml(xmlnode);
					machines.tag(in);
				}
			}
			xmlnode = xmlnode->next;
		}
		xmlFreeDoc(doc);
	}
}

void View::copy_from_topology(Data *data)
{
	Link *ln, *ln2;
	Device *r;
	Network *n;
	Machine *m;
	Interface *in;
	ViewItem *itm;

	empty();

	for (ln = data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		itm = new ViewItem();
		itm->ip.set(r->pingname);
		itm->x = r->x;
		itm->y = r->y;
		itm->shown = r->shown;
		itm->group3 = r->group;
		devices.tag(itm);
	}
	for (ln = data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		ln2 = n->interfaces.get_head();
		if (ln2!=NULL)
		{
			in = (Interface*) ln2->item;
			itm = new ViewItem();
			itm->ip.set(in->device->pingname);
			itm->iface = in->index;
			itm->x = n->x;
			itm->y = n->y;
			itm->shown = n->shown;
			itm->group = n->subtype;
			itm->position = n->position;
			itm->group2 = n->group;
			networks.tag(itm);
		}
	}
	for (ln = data->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		// only for visible machines (speedup)
		if (m->shown==TRUE)
		{
			itm = new ViewItem();
			itm->mac.set(m->mac);
			itm->x = m->x;
			itm->y = m->y;
			itm->shown = m->shown;
			machines.tag(itm);
		}
	}
}

void View::copy_to_topology(Data *data)
{
	Link *ln, *ln2, *ln3;
	ViewItem *itm;
	Device *r;
	Network *n;
	Machine *m;
	Interface *in;

	for (ln2 = data->devices.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		r = (Device*) ln2->item;
		for (ln = devices.get_head(); ln!=NULL; ln=ln->next)
		{
			itm = (ViewItem*) ln->item;
			if (r->pingname.compare(itm->ip)==0)
			{
				r->x = itm->x;
				r->y = itm->y;
				r->shown = itm->shown;
				r->group = itm->group3;
			}
		}
		for (ln = r->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			for (ln3 = networks.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				itm = (ViewItem*) ln3->item;
				if ((r->pingname.compare(itm->ip)==0) && (itm->iface==in->index))
				{
					in->network->x = itm->x;
					in->network->y = itm->y;
					in->network->shown = itm->shown;
					in->network->subtype = itm->group;
					in->network->position = itm->position;
					in->network->group = itm->group2;

					if (in->network->subtype==network_ring)
					{ in->network->w = 96*4; in->network->h = 96*4; }else
					if ((in->network->subtype!=network_hbus) &&
							(in->network->subtype!=network_vbus))
					{ in->network->w = 96; in->network->h = 96; }
				}
			}
		}
	}
	for (ln2 = data->machines.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		m = (Machine*) ln2->item;
		for (ln3 = machines.get_head(); ln3!=NULL; ln3=ln3->next)
		{
			itm = (ViewItem*) ln3->item;
			if (m->mac.compare(itm->mac)==1)
			{
				m->x = itm->x;
				m->y = itm->y;
				m->shown = itm->shown;
			}
		}
	}
}

